@@ -413,13 +413,10 @@ void labeldefaults(labelptr newlabel, u_char dopin, int x, int y)
    newlabel->string->data.font = areawin->psfont;
    newlabel->string->nextpart = NULL;
 
-   newlabel->pin = False;
-   if (areawin->schemon) {
-      newlabel->pin = dopin;
-      if (dopin == LOCAL) newlabel->color = LOCALPINCOLOR;
-      else if (dopin == GLOBAL) newlabel->color = GLOBALPINCOLOR;
-      else if (dopin == INFO) newlabel->color = INFOLABELCOLOR;
-   }
+   newlabel->pin = dopin;
+   if (dopin == LOCAL) newlabel->color = LOCALPINCOLOR;
+   else if (dopin == GLOBAL) newlabel->color = GLOBALPINCOLOR;
+   else if (dopin == INFO) newlabel->color = INFOLABELCOLOR;
 
    newlabel->justify = areawin->justify;
    newlabel->position.x = x;
@@ -525,7 +522,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 
    curlabel = TOLABEL(EDITPART);
 
-   if (curlabel == NULL || textpos <= 0) {
+   if (curlabel == NULL || curlabel->type != LABEL || textpos <= 0) {
       Wprintf("Error:  Bad label string");
       return FALSE;
    }
@@ -535,7 +532,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 
    UDrawTLine(curlabel);
 
-   if (r = isbound(keypressed, XCF_Text_Delete)) {
+   if (clientdata != NULL && keypressed == TEXT_DELETE) {
       if (textpos > 1) {
          int curloc, strpos;
          stringpart *strptr;
@@ -571,7 +568,7 @@ Boolean labeltext(int keypressed, char *clientdata)
          do_redraw = True;
       }
    }
-   else if (r = isbound(keypressed, XCF_Text_Delete_Param)) {
+   else if (clientdata != NULL && keypressed == TEXT_DEL_PARAM) {
       if (textpos > 1) {
          int curloc, strpos;
          stringpart *strptr;
@@ -588,7 +585,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 	 }
       }
    }
-   else if (r = isbound(keypressed, XCF_Text_Return)) {
+   else if (clientdata != NULL && keypressed == TEXT_RETURN) {
       Boolean hasstuff = False;   /* Check for null string */
       stringpart *tmppos;
 
@@ -682,8 +679,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 	 /* name is unique, change every pin in the matching symbol/	*/
 	 /* schematic to match the new text.				*/
 
-	 if ((areawin->schemon == True) && (curlabel->pin == LOCAL) && 
-		(topobject->symschem != NULL)) {
+	 if ((curlabel->pin == LOCAL) && (topobject->symschem != NULL)) {
 	    if ((findlabelcopy(curlabel, labelbuf) == NULL)
 			&& (findlabelcopy(curlabel, curlabel->string) == NULL)) {
 	       changeotherpins(curlabel, labelbuf);
@@ -705,13 +701,13 @@ Boolean labeltext(int keypressed, char *clientdata)
 
       return r;
    }
-   else if (r = isbound(keypressed, XCF_Text_Right)) {
+   else if (clientdata != NULL && keypressed == TEXT_RIGHT) {
       if (curpos != NULL) textpos++;
    }
-   else if (r = isbound(keypressed, XCF_Text_Left)) {
+   else if (clientdata != NULL && keypressed == TEXT_LEFT) {
       if (textpos > 1) textpos--;
    }
-   else if (r = isbound(keypressed, XCF_Text_Down)) {
+   else if (clientdata != NULL && keypressed == TEXT_DOWN) {
       while (curpos != NULL) {
 	 textpos++;
 	 curpos = findstringpart(textpos, &locpos, curlabel->string,
@@ -721,7 +717,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 	       break;
       }
    }
-   else if (r = isbound(keypressed, XCF_Text_Up)) {
+   else if (clientdata != NULL && keypressed == TEXT_UP) {
       while (textpos > 1) {
 	 textpos--;
 	 curpos = findstringpart(textpos, &locpos, curlabel->string,
@@ -732,11 +728,11 @@ Boolean labeltext(int keypressed, char *clientdata)
 	 }
       }
    }
-   else if (r = isbound(keypressed, XCF_Text_Home))
+   else if (clientdata != NULL && keypressed == TEXT_HOME)
       textpos = 1;
-   else if (r = isbound(keypressed, XCF_Text_End))
+   else if (clientdata != NULL && keypressed == TEXT_END)
       textpos = stringlength(curlabel->string, True, areawin->topinstance);
-   else if (r = isbound(keypressed, XCF_Text_Split)) {
+   else if (clientdata != NULL && keypressed == TEXT_SPLIT) {
       labelptr *newlabel;
       XPoint points[4], points1[4], points2[4];
 
@@ -781,7 +777,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 
    /* Write a font designator or other control into the string */
 
-   if (clientdata != NULL) {
+   else if (clientdata != NULL) {
       oparamptr ops;
       stringpart *newpart;
       Boolean errcond = False;
@@ -927,10 +923,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 
 void textreturn()
 {
-   int rkey;
-
-   rkey = firstbinding(XCF_Text_Return);
-   labeltext(rkey, NULL);
+   labeltext(TEXT_RETURN, (char *)1);
 }
 
 /*-------------------------------------*/
