@@ -347,28 +347,28 @@ short checkbounds()
 
    /* check window-to-user space */
 
-   lval = 2 * (long)((float) (areawin->width) / (*areawin->vscale)) +
-	(long)areawin->pcorner->x;
+   lval = 2 * (long)((float) (areawin->width) / areawin->vscale) +
+	(long)areawin->pcorner.x;
    if (lval != (long)((short)lval)) return -1;
-   lval = 2 * (long)((float) (areawin->height) / (*areawin->vscale)) +
-	(long)areawin->pcorner->y;
+   lval = 2 * (long)((float) (areawin->height) / areawin->vscale) +
+	(long)areawin->pcorner.y;
    if (lval != (long)((short)lval)) return -1;
 
    /* check user-to-window space */
 
-   lval = (long)((float)(topobject->bbox.lowerleft.x - areawin->pcorner->x) *
-	(*areawin->vscale));
+   lval = (long)((float)(topobject->bbox.lowerleft.x - areawin->pcorner.x) *
+	areawin->vscale);
    if (lval != (long)((short)lval)) return -1;
    lval = (long)areawin->height - (long)((float)(topobject->bbox.lowerleft.y -
-	areawin->pcorner->y) * (*areawin->vscale)); 
+	areawin->pcorner.y) * areawin->vscale); 
    if (lval != (long)((short)lval)) return -1;
    UTransformbyCTM(DCTM, &(topobject->bbox.lowerleft), &testpt, 1);
 
    lval = (long)((float)(topobject->bbox.lowerleft.x + topobject->bbox.width -
-	areawin->pcorner->x) * (*areawin->vscale));
+	areawin->pcorner.x) * areawin->vscale);
    if (lval != (long)((short)lval)) return -1;
    lval = (long)areawin->height - (long)((float)(topobject->bbox.lowerleft.y +
-	topobject->bbox.height - areawin->pcorner->y) * (*areawin->vscale)); 
+	topobject->bbox.height - areawin->pcorner.y) * areawin->vscale); 
    if (lval != (long)((short)lval)) return -1;
 
    return 0;
@@ -382,9 +382,9 @@ void window_to_user(short xw, short yw, XPoint *upt)
 {
   float tmpx, tmpy;
 
-  tmpx = (float)xw / (*areawin->vscale) + (float)areawin->pcorner->x;
-  tmpy = (float)(areawin->height - yw) / (*areawin->vscale) + 
-	(float)areawin->pcorner->y;
+  tmpx = (float)xw / areawin->vscale + (float)areawin->pcorner.x;
+  tmpy = (float)(areawin->height - yw) / areawin->vscale + 
+	(float)areawin->pcorner.y;
 
   tmpx += (tmpx > 0) ? 0.5 : -0.5;
   tmpy += (tmpy > 0) ? 0.5 : -0.5;
@@ -401,9 +401,9 @@ void user_to_window(XPoint upt, XPoint *wpt)
 {
   float tmpx, tmpy;
 
-  tmpx = (float)(upt.x - areawin->pcorner->x) * (*areawin->vscale);
-  tmpy = (float)areawin->height - (float)(upt.y - areawin->pcorner->y)
-	* (*areawin->vscale); 
+  tmpx = (float)(upt.x - areawin->pcorner.x) * areawin->vscale;
+  tmpy = (float)areawin->height - (float)(upt.y - areawin->pcorner.y)
+	* areawin->vscale; 
 
   tmpx += (tmpx > 0) ? 0.5 : -0.5;
   tmpy += (tmpy > 0) ? 0.5 : -0.5;
@@ -571,14 +571,14 @@ void UCopyCTM(fctm, tctm)
 
 void UMakeWCTM(Matrix *ctm)
 {
-   ctm->a *= (*areawin->vscale);
-   ctm->b *= (*areawin->vscale);
-   ctm->c = (ctm->c - (float)areawin->pcorner->x) * (*areawin->vscale);
+   ctm->a *= areawin->vscale;
+   ctm->b *= areawin->vscale;
+   ctm->c = (ctm->c - (float)areawin->pcorner.x) * areawin->vscale;
    
-   ctm->d *= -(*areawin->vscale);
-   ctm->e *= -(*areawin->vscale);
-   ctm->f = (float)areawin->height + ((float)areawin->pcorner->y - ctm->f) *
-	    (*areawin->vscale);
+   ctm->d *= -areawin->vscale;
+   ctm->e *= -areawin->vscale;
+   ctm->f = (float)areawin->height + ((float)areawin->pcorner.y - ctm->f) *
+	    areawin->vscale;
 }
 
 /*------------------------------------------------------------------------*/
@@ -1359,6 +1359,13 @@ void centerview(objinstptr tinst)
 	       / tobj->viewscale - width) / 2;
    tobj->pcorner.y = origin.y - (areawin->height
 	       / tobj->viewscale - height) / 2;
+
+   /* Copy new position values to the current window */
+
+   if (tobj == topobject) {
+      areawin->pcorner = tobj->pcorner;
+      areawin->vscale = tobj->viewscale;
+   }
 }
 
 /*-----------------------------------------------------------*/
@@ -1834,11 +1841,11 @@ void strokepath(XPoint *pathlist, short number, short style, float width)
       if (style & (DASHED | DOTTED)) {
          SetDashes(dpy, areawin->gc, 0, dashstring, 2);
          SetLineAttributes(dpy, areawin->gc, tmpwidth, LineOnOffDash,
-		CapButt, JoinBevel);
+		CapButt, JoinMiter);
       }
       else
          SetLineAttributes(dpy, areawin->gc, tmpwidth, LineSolid,
-		CapRound, JoinBevel);
+		(style & SQUARECAP) ? CapProjecting : CapRound, JoinMiter);
 
       /* draw the spline and close off if so specified */
       DrawLines(dpy, areawin->window, areawin->gc, pathlist,
@@ -2172,7 +2179,11 @@ void TopDoLatex()
 
    if (checklatex == FALSE) return;	/* No LaTeX labels to write */
 
-   sprintf(filename, "%s.tex", xobjs.pagelist[areawin->page]->filename);
+   if (xobjs.pagelist[areawin->page]->filename)
+      sprintf(filename, "%s.tex", xobjs.pagelist[areawin->page]->filename);
+   else
+      sprintf(filename, "%s.tex",
+		xobjs.pagelist[areawin->page]->pageinst->thisobject->name);
    f = fopen(filename, "w");
 
    fprintf(f, "%% XCircuit output \"%s\" for LaTeX input from %s.ps\n",
@@ -2185,8 +2196,14 @@ void TopDoLatex()
    fprintf(f, "\\def\\topbox#1{\\raisebox{-\\baselineskip}[0in][0in]{#1}}\n");
    fprintf(f, "\\def\\midbox#1{\\raisebox{-0.5\\baselineskip}[0in][0in]{#1}}\n");
    fprintf(f, "\\begin{flushleft}\n");
-   fprintf(f, "\\epsfig{file=%s.ps}\\\\\n",
-		xobjs.pagelist[areawin->page]->filename);
+   
+   /* Handle cases where the file might have a ".eps" extension.	*/
+   /* Thanks to Graham Sheward for pointing this out.			*/
+   filename[strlen(filename) - 4] = '\0';
+   if (strchr(filename, '.') == NULL)
+       sprintf(filename + strlen(filename), ".ps");
+
+   fprintf(f, "\\epsfig{file=%s}\\\\\n", filename);
 
    psscale = getpsscale(xobjs.pagelist[areawin->page]->outscale,
 		areawin->page);
