@@ -1511,8 +1511,7 @@ static PyObject *xc_set(PyObject *self, PyObject *args)
    }
 
    else if (!strcmp(sarg1, "xschema") || !strcmp(sarg1, "schema")) {
-      areawinschemon = a;
-      doxschema(OptionsEnableXSchemaButton, 1, NULL);
+      /* Do nothing---retained for backward compatibility only */
    }
 #ifdef HAVE_XPM
    else if (!strcmp(sarg1, "toolbar")) {
@@ -1655,12 +1654,15 @@ static PyObject *xc_bind(PyObject *self, PyObject *args)
 	 keywstate = -1;
 	 func = string_to_func(keyname, NULL);
       }
-      binding = binding_to_string(keywstate, func);
+      if (keywstate == -1)
+         binding = function_binding_to_string(0, func);
+      else
+         binding = key_binding_to_string(0, keywstate);
       retobj = PyString_FromString(binding);
       free(binding);
    }
    else {
-      if (add_keybinding(keyname, function) < 0) {
+      if (add_keybinding(0, keyname, function) < 0) {
 	 /* Function may be a Python function, not a C function */
          keywstate = string_to_key(keyname);
 	 sprintf(_STR2, "keydict[%d] = %s\n", keywstate, function);
