@@ -43,8 +43,7 @@ extern Tcl_Interp *xcinterp;
 #endif
 
 extern Globaldata xobjs;
-extern Clientdata areastruct;
-extern short	  eventmode;
+extern XCWindowData *areawin;
 extern int	  *appcolors;
 extern xcWidget     menuwidgets[];
 extern xcWidget	  netbutton;
@@ -81,45 +80,6 @@ void callwritenet(xcWidget w, pointertype mode, caddr_t calldata)
    }
 }
 
-/*--------------------------------------------------------------*/
-/* Enable schematic capture (set schemon and realize widgets)	*/
-/* If mode == 1, do a redraw (e.g., not called during a file	*/
-/* load)							*/
-/*--------------------------------------------------------------*/
-
-void doxschema(xcWidget w, pointertype mode, caddr_t calldata)
-{
-   Arg wargs[1];
-
-   if (areastruct.schemon) {
-      areastruct.schemon = False;
-#ifndef TCL_WRAPPER
-      XtUnmanageChild(wschema);
-      XtUnmanageChild(wsymb);
-      XtUnmanageChild(netbutton);
-
-      XtSetArg(wargs[0], XtNlabel, "Enable XSchema");
-      XtSetValues(OptionsEnableXSchemaButton, wargs, 1);
-#endif
-   }
-   else {
-      areastruct.schemon = True;
-#ifndef TCL_WRAPPER
-      XtManageChild(wschema);
-      XtManageChild(wsymb);
-      XtManageChild(netbutton);
-
-      XtSetArg(wargs[0], XtNlabel, "Disable XSchema");
-      XtSetValues(OptionsEnableXSchemaButton, wargs, 1);
-#endif
-   }
-
-   /* Redraw the screen, since this action may effect the presence of	*/
-   /* pins and schematic informational labels.				*/
-
-   if (mode) drawarea(NULL, NULL, NULL);
-}
-
 /*----------------------------------------------------------------------*/
 /* Find the page object and instance with the indicated name.		*/
 /*----------------------------------------------------------------------*/
@@ -229,7 +189,7 @@ int checkpagename(objectptr thispageobj)
 
 void startconnect(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
-   if (areastruct.selects > 0)
+   if (areawin->selects > 0)
       connectivity(button, clientdata, calldata);
 }
 
@@ -252,17 +212,17 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
    stringpart *ppin;
 
    /* erase any existing highlighted network */
-   highlightnetlist(topobject, areastruct.topinstance, 0);
+   highlightnetlist(topobject, areawin->topinstance, 0);
 
    seltop = (pushlistptr)malloc(sizeof(pushlist));
-   seltop->thisinst = areastruct.topinstance;
+   seltop->thisinst = areawin->topinstance;
    seltop->next = NULL;
 
    /* pick the first selection that looks like a valid network part */
 
-   if (areastruct.selects > 0) {
-      for (gsel = areastruct.selectlist; gsel < areastruct.selectlist +
-		areastruct.selects; gsel++) {
+   if (areawin->selects > 0) {
+      for (gsel = areawin->selectlist; gsel < areawin->selectlist +
+		areawin->selects; gsel++) {
 	 ggen = *(topobject->plist + *gsel);
          if (SELECTTYPE(gsel) == LABEL) {
 	    labelptr glab = SELTOLABEL(gsel);
@@ -274,8 +234,8 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 	 }
       }
    }
-   if ((areastruct.selects == 0) || (gsel == areastruct.selectlist
-		+ areastruct.selects)) {
+   if ((areawin->selects == 0) || (gsel == areawin->selectlist
+		+ areawin->selects)) {
       rselect = recurselect(POLYGON | LABEL | OBJINST, MODE_CONNECT, &seltop);
       /* Should look at the top scorer, when scoring has been implemented */
       if (rselect && (rselect->selects > 0)) {
@@ -299,7 +259,7 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
    if (ggen != NULL) {
       if (checkvalid(topobject) == -1) {
          destroynets(topobject);
-         createnets(areastruct.topinstance);
+         createnets(areawin->topinstance);
       }
       if ((netlist = is_resolved(&ggen, seltop, &nettop)) != NULL) {
          depth = pushnetwork(seltop, nettop);
@@ -323,7 +283,7 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 
 	 if (netlist->subnets == 0) {
 	    ppin = nettopin(netlist->net.id, pschem, NULL);
-            snew = textprint(ppin, areastruct.topinstance);
+            snew = textprint(ppin, areawin->topinstance);
 	    sprintf(_STR2, "Network is \"%s\" in %s", snew, nettop->name);
             free(snew);
 	 }
@@ -336,7 +296,7 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 	       netid = sbus->netid;
 	       subnetid = sbus->subnetid;
 	       ppin = nettopin(netid, pschem, NULL);
-               snew = textprintsubnet(ppin, areastruct.topinstance, subnetid);
+               snew = textprintsubnet(ppin, areawin->topinstance, subnetid);
 	       sprintf(sptr, "%s ", snew);
 	       sptr += strlen(snew) + 1;
 	       free(snew);
@@ -438,7 +398,7 @@ void dopintype(xcWidget w, pointertype mode, caddr_t calldata)
    char typestr[40];
    short savetype = -1;
 
-   if (areastruct.selects == 0) {
+   if (areawin->selects == 0) {
       Wprintf("Must first select a label to change type");
       return;
    }
@@ -459,8 +419,8 @@ void dopintype(xcWidget w, pointertype mode, caddr_t calldata)
          break;
    }
 
-   for (gsel = areastruct.selectlist; gsel < areastruct.selectlist +
-		areastruct.selects; gsel++)
+   for (gsel = areawin->selectlist; gsel < areawin->selectlist +
+		areawin->selects; gsel++)
       if (SELECTTYPE(gsel) == LABEL) {
 	 labelptr glab = SELTOLABEL(gsel);
 	 savetype = glab->pin;
@@ -693,7 +653,7 @@ int checkschem(objectptr thisobj, char *cname)
    objectptr *tlib;
    short i, j;
 
-   if ((areastruct.schemon == False) || (thisobj->symschem != NULL)) return 0;
+   if (thisobj->symschem != NULL) return 0;
 
    for (i = 0; i < xobjs.numlibs; i++) {
       for (j = 0; j < xobjs.userlibs[i].number; j++) {
@@ -721,7 +681,7 @@ int checksym(objectptr symobj, char *cname)
    short cpage;
    objectptr checkpage;
 
-   if (areastruct.schemon == False || symobj->symschem != NULL) return 0;
+   if (symobj->symschem != NULL) return 0;
 
    for (cpage = 0; cpage < xobjs.pages; cpage++) {
       if (xobjs.pagelist[cpage]->pageinst != NULL) {
@@ -790,7 +750,6 @@ void swapschem(int allow_create, int libnum)
    pushlistptr stacktop;
    short loclibnum = libnum;
 
-   if (areastruct.schemon == False) return;
    if (libnum == -1) loclibnum = USERLIB - LIBRARY;
 
    if (eventmode == MOVE_MODE || eventmode == COPY_MODE)
@@ -811,11 +770,11 @@ void swapschem(int allow_create, int libnum)
 	 /* Push the current instance onto the push stack */
 	 /* Change the page without destroying the pushlist */
 
-	 push_stack(&areastruct.stack, areastruct.topinstance);
-	 stacktop = areastruct.stack;
-	 areastruct.stack = NULL;
+	 push_stack(&areawin->stack, areawin->topinstance);
+	 stacktop = areawin->stack;
+	 areawin->stack = NULL;
          changepage(tpage);
-	 areastruct.stack = stacktop;
+	 areawin->stack = stacktop;
       }
       else {
 	 objectptr *newobject;
@@ -837,11 +796,13 @@ void swapschem(int allow_create, int libnum)
 	 /* Generate a library instance for this object and set the */
 	 /* top instance to point to it.			    */
 
-	 push_stack(&areastruct.stack, areastruct.topinstance);
-	 areastruct.topinstance = addtoinstlist(loclibnum, *newobject, FALSE);
+	 topobject->viewscale = areawin->vscale;
+	 topobject->pcorner = areawin->pcorner;
+	 push_stack(&areawin->stack, areawin->topinstance);
+	 areawin->topinstance = addtoinstlist(loclibnum, *newobject, FALSE);
 
 	 /* Generate the default bounding box for a size-zero object */
-	 calcbbox(areastruct.topinstance);
+	 calcbbox(areawin->topinstance);
       }
 
       /* set links between the two objects */
@@ -880,7 +841,7 @@ void swapschem(int allow_create, int libnum)
 	       (*pinlab)->color = lpin->color;
 	       (*pinlab)->rotation = 0;
 	       (*pinlab)->scale = 1.0;
-	       (*pinlab)->justify = areastruct.justify; 
+	       (*pinlab)->justify = areawin->justify; 
 	       (*pinlab)->position.x = 0;
 	       (*pinlab)->position.y = topobject->parts * (TEXTHEIGHT + 10);
 	       (*pinlab)->passed = NULL;
@@ -891,7 +852,7 @@ void swapschem(int allow_create, int libnum)
 	    }
          }
       }
-      calcbbox(areastruct.topinstance);
+      calcbbox(areawin->topinstance);
 
       /* Recreate the user library with the new symbol */
       if (savepage->schemtype != SYMBOL) composelib(loclibnum + LIBRARY);
@@ -901,10 +862,12 @@ void swapschem(int allow_create, int libnum)
       /* If symschem matches the last entry on the push stack, then we	*/
       /* pop; otherwise, we push.					*/
 
-      if (areastruct.stack && areastruct.stack->thisinst->thisobject
+      if (areawin->stack && areawin->stack->thisinst->thisobject
 			== topobject->symschem) {
-	 areastruct.topinstance = areastruct.stack->thisinst;
-	 pop_stack(&areastruct.stack);
+	 topobject->viewscale = areawin->vscale;
+	 topobject->pcorner = areawin->pcorner;
+	 areawin->topinstance = areawin->stack->thisinst;
+	 pop_stack(&areawin->stack);
       }
       else {
 	 int p;
@@ -940,8 +903,10 @@ void swapschem(int allow_create, int libnum)
 	    }
          }
 	       
-	 push_stack(&areastruct.stack, areastruct.topinstance);
-	 areastruct.topinstance = syminst;
+	 topobject->viewscale = areawin->vscale;
+	 topobject->pcorner = areawin->pcorner;
+	 push_stack(&areawin->stack, areawin->topinstance);
+	 areawin->topinstance = syminst;
       }
    }
 
@@ -979,10 +944,6 @@ void dobeforeswap(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    buttonsave *popdata = (buttonsave *)malloc(sizeof(buttonsave));
 
-   /* Check if this function is valid */
-
-   if (areastruct.schemon == False) return;
-
    /* Check for requirement to change the name before creating the symbol */
 
    if ((topobject->symschem == NULL) && (topobject->schemtype == PRIMARY)
