@@ -65,6 +65,11 @@ typedef struct {
    } save;
 } editelement;
 
+typedef struct {
+   genericptr element;		/* element modified */
+   float scale;			/* old scale value */
+} scaleinfo;
+
 /*----------------------------------------------------------------------*/
 /* Externally declared variables					*/
 /*----------------------------------------------------------------------*/
@@ -327,15 +332,17 @@ void undelete_one_element(objinstptr thisinst, genericptr thiselem)
 void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 {
    va_list args;
-   int usize, drawmode, npage, opage, *idata, snum, deltax, deltay, dir;
+   int usize, drawmode, nval, oval, *idata, snum, deltax, deltay, dir;
    short *slist;
    objectptr delobj;
    objinstptr newinst;
    Undoptr newrecord;
    uselection *srec;
    editelement *erec;
+   scaleinfo *escale;
    genericptr egen, newgen;
    XPoint *fpoint;
+   double scale;
 
    /* Do not register new events while running undo/redo actions! */
    if (eventmode == UNDO_MODE) return;
@@ -391,14 +398,14 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 
       case XCF_Page:
 	 /* 2 args:							*/
-	 /*	opage = original page (int)				*/
-	 /*	npage = new page (int)					*/
-	 opage = va_arg(args, int);
-	 npage = va_arg(args, int);
+	 /*	oval = original integer value				*/
+	 /*	nval = new integer value				*/
+	 oval = va_arg(args, int);
+	 nval = va_arg(args, int);
 	 idata = (int *)malloc(sizeof(int));
-	 *idata = npage;
+	 *idata = nval;
 	 newrecord->undodata = (char *)idata;
-	 newrecord->idata = opage;
+	 newrecord->idata = oval;
 	 break;
 
       case XCF_Pop:
@@ -469,6 +476,32 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	 newrecord->undodata = (char *)erec;
 	 break;
 
+      case XCF_ChangeStyle:
+      case XCF_Justify:
+	 /* 2 args:							*/
+	 /*	egen = element that was changed (with new value)	*/
+	 /*	oval = old style value					*/
+	 egen = va_arg(args, genericptr);
+	 oval = va_arg(args, int);
+	 newrecord->undodata = (char *)egen;
+	 newrecord->idata = oval;
+	 break;
+
+      case XCF_Scale:
+	 /* 2 args:							*/
+	 /*	egen = element that was changed (with new value)	*/
+	 /*	ofloat = old float value				*/
+
+	 egen = va_arg(args, genericptr);
+	 scale = va_arg(args, double); 		/* warning! only takes "double"! */
+
+	 escale = (scaleinfo *)malloc(sizeof(scaleinfo));
+	 escale->element = egen;
+	 escale->scale = (float)scale;
+
+	 newrecord->undodata = (char *)escale;
+	 break;
+
       case XCF_Flip_X: case XCF_Flip_Y:
 	 /* 1 arg:							*/
 	 /*	fpoint = point of flip (XPoint *)			*/
@@ -519,10 +552,18 @@ short undo_one_action()
    objinstptr thisinst;
    uselection *srec;
    editelement *erec;
+   scaleinfo *escale;
    short *slist;
    XPoint *delta;
    int i, snum, npage, opage;
+   float fnum;
    genericptr egen;
+   labelptr thislabel;
+   pathptr thispath;
+   polyptr thispoly;
+   arcptr thisarc;
+   splineptr thisspline;
+   graphicptr thisgraphic;
 
    /* Undo the recorded action and shift the undo record pointer.	*/
 
@@ -598,6 +639,14 @@ short undo_one_action()
 	 newpage(thisrecord->idata);
 	 break;
 
+      case XCF_Justify:
+	 thislabel = (labelptr)(thisrecord->undodata);
+	 snum = thisrecord->idata;
+	 thisrecord->idata = thislabel->justify;
+	 thislabel->justify = snum;
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
       case XCF_Select:
 
 	 /* If there was a previous selection in the undo list,	*/
@@ -703,6 +752,75 @@ short undo_one_action()
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
+      case XCF_ChangeStyle:
+	 /* Style changes */
+	 egen = (genericptr)thisrecord->undodata;
+	 snum = thisrecord->idata;
+	 switch(egen->type) {
+	    case PATH:
+	       thispath = (pathptr)(thisrecord->undodata);
+	       thisrecord->idata = thispath->style;
+	       thispath->style = snum;
+	       break;
+	    case POLYGON:
+	       thispoly = (polyptr)(thisrecord->undodata);
+	       thisrecord->idata = thispoly->style;
+	       thispoly->style = snum;
+	       break;
+	    case ARC:
+	       thisarc = (arcptr)(thisrecord->undodata);
+	       thisrecord->idata = thisarc->style;
+	       thisarc->style = snum;
+	       break;
+	    case SPLINE:
+	       thisspline = (splineptr)(thisrecord->undodata);
+	       thisrecord->idata = thisspline->style;
+	       thisspline->style = snum;
+	       break;
+	 }
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
+      case XCF_Scale:
+	 escale = (scaleinfo *)thisrecord->undodata;
+	 egen = escale->element;
+	 fnum = escale->scale;
+	 switch(egen->type) {
+	    case PATH:
+	       thispath = (pathptr)egen;
+	       escale->scale = thispath->width;
+	       thispath->width = fnum;
+	       break;
+	    case POLYGON:
+	       thispoly = (polyptr)egen;
+	       escale->scale = thispoly->width;
+	       thispoly->width = fnum;
+	       break;
+	    case ARC:
+	       thisarc = (arcptr)egen;
+	       escale->scale = thisarc->width;
+	       thisarc->width = fnum;
+	       break;
+	    case SPLINE:
+	       thisspline = (splineptr)egen;
+	       escale->scale = thisspline->width;
+	       thisspline->width = fnum;
+	       break;
+	    case OBJINST:
+	       thisinst = (objinstptr)egen;
+	       escale->scale = thisinst->scale;
+	       thisinst->scale = fnum;
+	       break;
+	    case GRAPHIC:
+	       thisgraphic = (graphicptr)egen;
+	       escale->scale = thisgraphic->scale;
+	       thisgraphic->scale = fnum;
+	       thisgraphic->valid = FALSE;
+	       break;
+	 }
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
       case XCF_Flip_X:
 	 areawin->save = *((XPoint *)thisrecord->undodata);
 	 elementflip();
@@ -765,7 +883,16 @@ short redo_one_action()
    XPoint *delta;
    uselection *srec;
    editelement *erec;
+   scaleinfo *escale;
    int i, tpage, snum;
+   float fnum;
+   labelptr thislabel;
+   arcptr thisarc;
+   pathptr thispath;
+   splineptr thisspline;
+   polyptr thispoly;
+   graphicptr thisgraphic;
+   objinstptr thisinst;
 
    /* Undo the recorded action and shift the undo record pointer.	*/
 
@@ -804,6 +931,14 @@ short redo_one_action()
 	 newpage(*((int *)thisrecord->undodata));
 	 break;
 
+      case XCF_Justify:
+	 thislabel = (labelptr)(thisrecord->undodata);
+	 snum = thisrecord->idata;
+	 thisrecord->idata = thislabel->justify;
+	 thislabel->justify = snum;
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
       case XCF_Pop:
 	 popobject(areawin->area, 0, NULL);
 	 break;
@@ -922,6 +1057,75 @@ short redo_one_action()
 	 elementvflip();
 	 break;
 
+      case XCF_ChangeStyle:
+	 /* Style changes */
+	 egen = (genericptr)thisrecord->undodata;
+	 snum = thisrecord->idata;
+	 switch(egen->type) {
+	    case PATH:
+	       thispath = (pathptr)(thisrecord->undodata);
+	       thisrecord->idata = thispath->style;
+	       thispath->style = snum;
+	       break;
+	    case POLYGON:
+	       thispoly = (polyptr)(thisrecord->undodata);
+	       thisrecord->idata = thispoly->style;
+	       thispoly->style = snum;
+	       break;
+	    case ARC:
+	       thisarc = (arcptr)(thisrecord->undodata);
+	       thisrecord->idata = thisarc->style;
+	       thisarc->style = snum;
+	       break;
+	    case SPLINE:
+	       thisspline = (splineptr)(thisrecord->undodata);
+	       thisrecord->idata = thisspline->style;
+	       thisspline->style = snum;
+	       break;
+	 }
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
+      case XCF_Scale:
+	 escale = (scaleinfo *)thisrecord->undodata;
+	 egen = escale->element;
+	 fnum = escale->scale;
+	 switch(egen->type) {
+	    case PATH:
+	       thispath = (pathptr)egen;
+	       escale->scale = thispath->width;
+	       thispath->width = fnum;
+	       break;
+	    case POLYGON:
+	       thispoly = (polyptr)egen;
+	       escale->scale = thispoly->width;
+	       thispoly->width = fnum;
+	       break;
+	    case ARC:
+	       thisarc = (arcptr)egen;
+	       escale->scale = thisarc->width;
+	       thisarc->width = fnum;
+	       break;
+	    case SPLINE:
+	       thisspline = (splineptr)egen;
+	       escale->scale = thisspline->width;
+	       thisspline->width = fnum;
+	       break;
+	    case OBJINST:
+	       thisinst = (objinstptr)egen;
+	       escale->scale = thisinst->scale;
+	       thisinst->scale = fnum;
+	       break;
+	    case GRAPHIC:
+	       thisgraphic = (graphicptr)egen;
+	       escale->scale = thisgraphic->scale;
+	       thisgraphic->scale = fnum;
+	       thisgraphic->valid = FALSE;
+	       break;
+	 }
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
       case XCF_Rotate:
 	 areawin->save = *((XPoint *)thisrecord->undodata);
 	 elementrotate(thisrecord->idata);
@@ -1059,6 +1263,8 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 	 break;
 
       case XCF_Push:
+      case XCF_ChangeStyle:
+      case XCF_Justify:
 	 /* Do nothing --- undodata points to a valid element */
 	 break;
 
