@@ -128,15 +128,16 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 
 /* Names used for convenience throughout the source code */
 
-#define topobject	areastruct.topinstance->thisobject
-#define hierobject	areastruct.hierstack->thisinst->thisobject
+#define eventmode	areawin->event_mode
+#define topobject	areawin->topinstance->thisobject
+#define hierobject	areawin->hierstack->thisinst->thisobject
 
 #define PLIST_INCR(a) \
 	(a)->plist = (genericptr *) realloc ((a)->plist, \
 	((a)->parts + 1) * sizeof(genericptr))
 
 #define ENDPART		topobject->plist + topobject->parts
-#define EDITPART	topobject->plist + areastruct.editpart
+#define EDITPART	topobject->plist + areawin->editpart
 
 /* Type checks (argument "a" is type genericptr) */
 
@@ -165,7 +166,7 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 
 /* conversions from a selection to a specific type */
 
-#define SELTOGENERICPTR(a) ((areastruct.hierstack == NULL) ? \
+#define SELTOGENERICPTR(a) ((areawin->hierstack == NULL) ? \
 			(topobject->plist + *(a)) : \
 			(hierobject->plist + *(a)))
 
@@ -363,6 +364,19 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define PARAM_START	17  /* bounds a parameter; data = param key	*/
 #define PARAM_END	18  /* bounds a parameter; no data 		*/
 
+/* Actions translated to keystates */
+
+#define TEXT_RETURN	19
+#define TEXT_HOME	20
+#define TEXT_END	21
+#define TEXT_SPLIT	22
+#define TEXT_DOWN	23
+#define TEXT_UP		24
+#define TEXT_LEFT	25
+#define TEXT_RIGHT	26
+#define TEXT_DELETE	27
+#define TEXT_DEL_PARAM	28
+
 #define SPECIAL		63  /* used only when called from menu		*/
 #define NULL_TYPE	255 /* used as a placeholder			*/
 
@@ -408,6 +422,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define FILLSOLID 224  /* = 32 + 64 + 128 */
 #define OPAQUE 256
 #define BBOX 512
+#define SQUARECAP 1024
 
 /*----------------------------------------------------------------------*/
 /* Box edit styles							*/
@@ -495,7 +510,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define WAITFOR		appcursors[9]
 #define HAND		appcursors[10]
 
-#define DEFAULTCURSOR	(*areastruct.defaultcursor)
+#define DEFAULTCURSOR	(*areawin->defaultcursor)
 
 /*----------------------------------------------------------------------*/
 /* selection-mechanism structures					*/
@@ -776,7 +791,7 @@ typedef struct _selection {
 #define select_element(a) recurse_select_element(a, UNDO_MORE) 
 #define select_add_element(a) recurse_select_element(a, UNDO_DONE) 
 
-#define easydraw(a, b)	 geneasydraw(a, b, topobject, areastruct.topinstance)
+#define easydraw(a, b)	 geneasydraw(a, b, topobject, areawin->topinstance)
 
 /*----------------------------------------------------------------------*/
 /* Netlist structures for schematic capture				*/
@@ -944,7 +959,7 @@ typedef struct _matrix {
 /* Some convenience functions for matrix manipulation			*/
 /*----------------------------------------------------------------------*/
 
-#define DCTM  areastruct.MatStack
+#define DCTM  areawin->MatStack
 
 /*----------------------------------------------------------------------*/
 /* button tap/press definitions 					*/
@@ -1091,19 +1106,19 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 #define XTopSetForeground(a) if (a == DEFAULTCOLOR) SetForeground(dpy, \
-	areastruct.gc, FOREGROUND); else SetForeground(dpy, areastruct.gc, a)
+	areawin->gc, FOREGROUND); else SetForeground(dpy, areawin->gc, a)
 
 #define XSetXORFg(a,b) if (a == DEFAULTCOLOR) SetForeground(dpy, \
-	areastruct.gc, FOREGROUND ^ b); else SetForeground(dpy, areastruct.gc,\
+	areawin->gc, FOREGROUND ^ b); else SetForeground(dpy, areawin->gc,\
 	a ^ b)
 
-#define XcSetFunction(z) SetFunction(dpy, areastruct.gc, z); \
-	areastruct.gctype = z
+#define XcSetFunction(z) SetFunction(dpy, areawin->gc, z); \
+	areawin->gctype = z
 
-#define XcSetForeground(z) XTopSetForeground(z); areastruct.gccolor = \
+#define XcSetForeground(z) XTopSetForeground(z); areawin->gccolor = \
 	((z) == DEFAULTCOLOR) ? FOREGROUND : (z)
 
-#define XcSetXORFg(y,z) XSetXORFg(y,z); areastruct.gccolor = \
+#define XcSetXORFg(y,z) XSetXORFg(y,z); areawin->gccolor = \
 	((y) == DEFAULTCOLOR) ? (FOREGROUND ^ z) : (y ^ z)
 
 /*----------------------------------------------------------------------*/
@@ -1151,6 +1166,7 @@ typedef struct {
 typedef struct _keybinding *keybindingptr;
 
 typedef struct _keybinding {
+   xcWidget window;		/* per-window function, or NULL */
    int keywstate;
    int function;
    short value;
@@ -1167,7 +1183,7 @@ enum {
    XCF_ENDDATA = -2,		XCF_SPACER  /* -1 */,
    XCF_Page	 /* 0 */,	XCF_Justify /* 1 */,
    XCF_Superscript /* 2 */,	XCF_Subscript /* 3 */,
-   XCF_Normalscript /* 4 */,	XCF_Nextfont /* 5 */,
+   XCF_Normalscript /* 4 */,	XCF_Font /* 5 */,
    XCF_Boldfont /* 6 */,	XCF_Italicfont /* 7 */,
    XCF_Normalfont /* 8 */,	XCF_Underline /* 9 */,
    XCF_Overline /* 10 */,	XCF_ISO_Encoding /* 11 */,
@@ -1219,7 +1235,8 @@ enum {
    XCF_Finish /* 102 */,	XCF_Cancel_Last /* 103 */,
    XCF_Sim /* 104 */,		XCF_SPICE /* 105 */,
    XCF_PCB /* 106 */,		XCF_SPICEflat /* 107 */,
-   XCF_Text_Delete_Param /* 108 */,
+   XCF_Color /* 108 */,		XCF_Text_Delete_Param /* 109 */,
+   XCF_Scale /* 110 */,		XCF_ChangeStyle /* 111 */,
    NUM_FUNCTIONS
 };
 
@@ -1265,7 +1282,7 @@ typedef struct {
    /* widgets and X11 parameters */
    xcWidget	area;
    xcWidget	scrollbarh, scrollbarv;
-   Window	areawin;
+   Window	window;
    GC 		gc;
    int		gccolor, gctype;
    xcIntervalId time_id;
@@ -1273,8 +1290,8 @@ typedef struct {
    /* global page parameters */
    short	width, height;
    short	page;
-   float	*vscale;	/* pointer to proper scale */
-   XPoint	*pcorner;	/* pointer to page position */
+   float	vscale;		/* proper scale */
+   XPoint	pcorner;	/* page position */
 
    /* global option defaults */
    float	textscale;
@@ -1283,8 +1300,8 @@ typedef struct {
    short	psfont;
    short	justify;
    u_short	style;
-   short	filter;		/* selection filter */
    int		color;
+   short	filter;		/* selection filter */
    Boolean	manhatn;
    Boolean	boxedit;
    Boolean	snapto;
@@ -1294,11 +1311,10 @@ typedef struct {
    Boolean	axeson;
    Boolean	invert;
    Boolean	mapped;		/* indicates if window is drawable */
-   Boolean	filefilter;	/* Is the file list filtered? */
    char		buschar;	/* Character indicating vector notation */
    Boolean	editinplace;
-   Boolean	schemon;
    Boolean	pinpointon;
+   Boolean	attachto;
 #ifdef HAVE_XPM
    Boolean	toolbar_on;
 #endif
@@ -1313,11 +1329,12 @@ typedef struct {
    Matrixptr	MatStack;
    pushlistptr	stack;
    pushlistptr	hierstack;
+   short	event_mode;
    short	editcycle;
    short	editpart, editsubpart;
    char		*lastbackground;
    Cursor	*defaultcursor;
-} Clientdata;
+} XCWindowData, *XCWindowDataPtr;
 
 /* Record for undo function */ 
 
@@ -1360,6 +1377,7 @@ typedef struct {
    xcIntervalId	timeout_id;
    int		save_interval;
    short	do_replace;
+   Boolean	filefilter;	/* Is the file list filtered? */
    u_short	new_changes;
    short	numlibs;
    short	pages;
@@ -1371,6 +1389,8 @@ typedef struct {
    objinstptr   *libtop;
    Imagedata	*imagelist;
    short	images;
+   XCWindowData *windowlist;
+   short	windows;
 } Globaldata;
 
 /*----------------------------------------------------------------------*/
